@@ -67,8 +67,10 @@ struct _PersonalGprsMonPrivate
 	
 	GConfClient *client;
 	gint refresh;
+	gint refreshSeconds;
 	gboolean binaryPrefix;
 	gboolean showResetDate;
+	gdouble version;
 	
 	guint eventID;
 };
@@ -93,7 +95,7 @@ void personal_gprs_mon_read_settings ( PersonalGprsMon *self )
 
         self->priv->refresh = g_key_file_get_integer (keyFile, "config", "refresh", &error);
         if (error) {
-            self->priv->refresh = 5;
+            self->priv->refresh = 1;
             g_error_free (error);
             error = NULL;
         }
@@ -109,11 +111,47 @@ void personal_gprs_mon_read_settings ( PersonalGprsMon *self )
 			g_error_free (error);
 			error = NULL;
 		}
+		// Introduced in version 0.4-1 to allow change in refresh intervals
+		self->priv->version = g_key_file_get_double (keyFile, "config", "version", &error);
+		if (error) {
+			self->priv->version = 0.3;
+			g_error_free (error);
+			error = NULL;
+		}
     } else {
-        self->priv->refresh = 5;
+        self->priv->refresh = 1;
 		self->priv->binaryPrefix = FALSE;
 		self->priv->showResetDate = FALSE;
+		self->priv->version = 0.3;
     }
+	
+	if (self->priv->version < 0.4) {
+		self->priv->refresh = 1;
+	}
+	
+	switch (self->priv->refresh) {
+		case 0:
+			self->priv->refreshSeconds = 0;
+			break;
+		case 1:
+			self->priv->refreshSeconds = 5;
+			break;
+		case 2:
+			self->priv->refreshSeconds = 30;
+			break;
+		case 3:
+			self->priv->refreshSeconds = 60;
+			break;
+		case 4:
+			self->priv->refreshSeconds = 300;
+			break;
+		case 5:
+			self->priv->refreshSeconds = 600;
+			break;
+		default:
+			self->priv->refreshSeconds = 5;
+			break;
+	}
 
     g_key_file_free (keyFile);
     g_free (filename);
@@ -136,6 +174,7 @@ void personal_gprs_mon_write_settings (PersonalGprsMon *self)
 	g_key_file_set_integer (keyFile, "config", "refresh", self->priv->refresh);
 	g_key_file_set_boolean (keyFile, "config", "binaryPrefix", self->priv->binaryPrefix);
 	g_key_file_set_boolean (keyFile, "config", "showResetDate", self->priv->showResetDate);
+	g_key_file_set_double (keyFile, "config", "version", 0.4);
 	
 	filename = g_strconcat (HOME_DIR, "/.personal_gprs_mon", NULL);
 	fileData = g_key_file_to_data (keyFile, &size, NULL);
@@ -147,9 +186,17 @@ void personal_gprs_mon_write_settings (PersonalGprsMon *self)
 	g_free (filename);
 }
 
+void personal_gprs_mon_button_release (GtkWidget *widget, GdkEventButton *event, PersonalGprsMon *self)
+{
+	//g_warning ("personal_gprs_mon_button_release");
+	personal_gprs_mon_update_content (self);
+}
+
 void personal_gprs_mon_content_create (PersonalGprsMon *self)
 {
 	//g_warning ("personal_gprs_mon_content_create");
+	GtkWidget *event = gtk_event_box_new ();
+	gtk_event_box_set_visible_window (GTK_EVENT_BOX (event), FALSE);
 	self->priv->contents = gtk_vbox_new (FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (self->priv->contents), 0);
 	GtkSizeGroup *group = GTK_SIZE_GROUP (gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL));
@@ -211,7 +258,10 @@ void personal_gprs_mon_content_create (PersonalGprsMon *self)
 	gtk_box_pack_start (GTK_BOX (self->priv->contents), uploadBox, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (self->priv->contents), downloadBox, FALSE, FALSE, 0);	
 	gtk_box_pack_start (GTK_BOX (self->priv->contents), self->priv->resetBox, FALSE, FALSE, 0);
-    gtk_box_pack_start (GTK_BOX (self->priv->homeWidget), GTK_WIDGET (self->priv->contents), FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (event), self->priv->contents);
+    gtk_box_pack_start (GTK_BOX (self->priv->homeWidget), event, FALSE, FALSE, 0);
+	
+	g_signal_connect (event, "button-release-event", G_CALLBACK (personal_gprs_mon_button_release), self);
 	
     gtk_widget_show_all (self->priv->homeWidget);
 	if (!self->priv->showResetDate) {
@@ -254,7 +304,11 @@ gboolean personal_gprs_mon_update_content (PersonalGprsMon *self)
 	
 	upload = g_ascii_strtoll (uploadString, NULL, 10);
 	download = g_ascii_strtoll (downloadString, NULL, 10);
-	resetTime = g_ascii_strtoll (resetString, NULL, 10);
+	if (resetString != NULL) {
+		resetTime = g_ascii_strtoll (resetString, NULL, 10);
+	} else {
+		resetTime = 0;
+	}
 	
 	gdouble uploadSize;
 	gdouble downloadSize;
@@ -325,7 +379,11 @@ gboolean personal_gprs_mon_update_content (PersonalGprsMon *self)
 	g_date_set_time_val (date, &timeVal);
 	gchar dateString[256];
 	g_date_strftime (dateString, 256, "%a, %d %b %Y", date);
-	gtk_label_set_text (GTK_LABEL (self->priv->resetContent), dateString);
+	if (resetTime == 0) {
+		gtk_label_set_text (GTK_LABEL (self->priv->resetContent), "Never");
+	} else {
+		gtk_label_set_text (GTK_LABEL (self->priv->resetContent), dateString);
+	}
 	
 	gtk_label_set_text (GTK_LABEL (self->priv->uploadContent), g_strconcat (g_strdup_printf ("%.3f", uploadSize), uploadUnits, NULL));
 	gtk_label_set_text (GTK_LABEL (self->priv->downloadContent), g_strconcat (g_strdup_printf ("%.3f", downloadSize), downloadUnits, NULL));
@@ -354,17 +412,17 @@ void personal_gprs_mon_settings (HDHomePluginItem *hitem, PersonalGprsMon *self)
 	
 	GtkWidget *selector = hildon_touch_selector_new_text ();
 	
-	int i;
-	for (i=0; i<=60; i++) {
-		gchar *text = g_strdup_printf ("%d seconds(s)", i);
-		hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector), text);
-		g_free (text);
-	}
-	self->priv->refresh;
+	hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector), "only when switching desktops");
+	hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector), "every 5 seconds");
+	hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector), "every 30 seconds");
+	hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector), "every 1 minute");
+	hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector), "every 5 minutes");
+	hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector), "every 10 minutes");
+		
 	hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (selector), 0, self->priv->refresh);
 	
 	GtkWidget *refreshButton = hildon_picker_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT, HILDON_BUTTON_ARRANGEMENT_HORIZONTAL);
-	hildon_button_set_title (HILDON_BUTTON (refreshButton), "Update every");
+	hildon_button_set_title (HILDON_BUTTON (refreshButton), "Update");
 	hildon_button_add_title_size_group (HILDON_BUTTON (refreshButton), group);
 	hildon_button_set_alignment (HILDON_BUTTON (refreshButton), 0, 0.5, 0, 0);
 
@@ -372,8 +430,8 @@ void personal_gprs_mon_settings (HDHomePluginItem *hitem, PersonalGprsMon *self)
                                      HILDON_TOUCH_SELECTOR (selector));
 	gtk_container_add (GTK_CONTAINER (content_area), refreshButton);
 	
-	GtkWidget *refreshLabel = gtk_label_new ("'0' will refresh only when switching to the desktop");
-	gtk_label_set_markup (GTK_LABEL (refreshLabel), "<small>'0' will refresh only when switching to the desktop</small>");
+	GtkWidget *refreshLabel = gtk_label_new ("The widget will always refresh when you press it");
+	gtk_label_set_markup (GTK_LABEL (refreshLabel), "<small>The widget will always refresh when you press it</small>");
 	gtk_container_add (GTK_CONTAINER (content_area), refreshLabel);
 	
 	GtkWidget *hbox = gtk_hbox_new (FALSE, 0);
@@ -401,6 +459,37 @@ void personal_gprs_mon_settings (HDHomePluginItem *hitem, PersonalGprsMon *self)
 		self->priv->binaryPrefix = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (binaryPrefix));
 		self->priv->showResetDate = hildon_check_button_get_active (HILDON_CHECK_BUTTON (showResetDate));
 		
+		switch (self->priv->refresh) {
+		case 0:
+			self->priv->refreshSeconds = 0;
+			break;
+		case 1:
+			self->priv->refreshSeconds = 5;
+			break;
+		case 2:
+			self->priv->refreshSeconds = 30;
+			break;
+		case 3:
+			self->priv->refreshSeconds = 60;
+			break;
+		case 4:
+			self->priv->refreshSeconds = 300;
+			break;
+		case 5:
+			self->priv->refreshSeconds = 600;
+			break;
+		default:
+			self->priv->refreshSeconds = 5;
+			break;
+		}
+		
+		if (self->priv->eventID) {
+			g_source_remove (self->priv->eventID);
+		}
+		if (self->priv->refreshSeconds > 0) {
+			self->priv->eventID = g_timeout_add (self->priv->refreshSeconds*1000, (GSourceFunc)personal_gprs_mon_update_content, self);
+		}
+		
 		personal_gprs_mon_write_settings (self);
 		personal_gprs_mon_update_content (self);
 	}
@@ -416,8 +505,8 @@ static void personal_gprs_mon_check_desktop (GObject *gobject, GParamSpec *pspec
 	if (status) {
 		personal_gprs_mon_update_content (self);
 		if (self->priv->eventID == 0) {
-			if (self->priv->refresh > 0) {
-				self->priv->eventID = g_timeout_add (self->priv->refresh*1000, (GSourceFunc)personal_gprs_mon_update_content, self);
+			if (self->priv->refreshSeconds > 0) {
+				self->priv->eventID = g_timeout_add (self->priv->refreshSeconds*1000, (GSourceFunc)personal_gprs_mon_update_content, self);
 			}
 		}
 	} else if (self->priv->eventID != 0) {
